@@ -94,7 +94,6 @@ rollcall <- function(data,
       rownames(legis.data) <- nm
   }
 
-  ## roll call votes attributes
   if(!is.null(vote.data)){
     if(nrow(vote.data)!=ncol(v))
       stop("rows in vote.data does not match number of votes in roll call matrix")
@@ -183,10 +182,13 @@ print.rollcall <- function(x,print.votes=FALSE, ...){
 
 ## check Votes
 checkVotes <- function(object,codes=object$codes){
-  if(class(object)=="rollcall")
+  if(inherits(object, "rollcall")){
     mat <- object$votes
-  if(class(object)=="matrix")
-    mat <- object
+  } else {
+    if("matrix" %in% class(object)) {
+      mat <- object
+    }
+  }
   if(is.null(codes))
     stop("checkVotes: no codes supplied")
   
@@ -272,6 +274,7 @@ convertCodes <- function(object,codes=object$codes){
   tmp <- matrix(-999,
                 dim(object$votes)[1],
                 dim(object$votes)[2])
+  dimnames(tmp) <- dimnames(object$votes)
   tmp[object$votes %in% theCodes$yea] <- 1
   tmp[object$votes %in% theCodes$nay] <- 0
   if(!is.null(theCodes$missing)){
@@ -356,7 +359,7 @@ lopLook <- function(margins,cutOff){
 vectorRepresentation <- function(object,
                                  dropList=list(codes=c("missing",
                                                  "notInLegis"))){
-  if(class(object)!="rollcall")
+  if(!inherits(object, "rollcall"))
     stop("vectorRepresentation only defined for objects of class rollcall")
   if(is.null(object$codes))
     stop("no rollcall codes")
@@ -433,7 +436,7 @@ summary.rollcall <- function(object,
                              verbose=FALSE,
                              debug=FALSE,
                              ...){
-  if(class(object)!="rollcall")
+  if(!inherits(object, "rollcall"))
     stop("summary.rollcall only operates on objects of class rollcall")
 
   mc <- match.call()   ## how were we called
@@ -549,7 +552,7 @@ printDropTab <- function(x){
 }
 
 print.summary.rollcall <- function(x, digits=1, ...){
-  if(class(x)!="summary.rollcall")
+  if(!inherits(x, "summary.rollcall"))
     stop("print.summary.rollcall only defined for objects of class summary.rollcall")
     
   rcObj <- x$call$object
